/* 
  DualVoltage Sensor Wrapper for Teensy 3.x
  -------------------------------------------------------------
  
  Copyright 2016 By Bernd Wokoeck
  
  Version history:
  1.00   02/21/2016  created

  Dependencies:
  - ADC library for Teensy https://github.com/pedvide/ADC

**************************************************************/
#if ARDUINO >= 100
 #include "Arduino.h"
#else
 #include "WProgram.h"
#endif

class DualVoltageSensor
{
public:
  DualVoltageSensor()  { m_channel = 0; m_status = IDLE; m_period = 0; m_endTime = 0; m_val = -1; m_pin1 = m_pin2 = -1; m_factor = 1.0f; }
  ~DualVoltageSensor() {}

  void Init( int pin1, int pin2, float conv1, float conv2, int period = 100 ); // conv = conversion factor from binary to volt: default period 100ms = 10 Hz 
  void DoDualVoltageSensor();

  long  GetDualVoltage() { return m_val; }

protected:

  enum enStatus
  {
    IDLE       = 0, // measurement can be started 
    CONVERTING = 1, // wait for result
    SLEEPING   = 2, // wait until next measurement, determines rate
  };

  int        m_pin1;
  int        m_pin2;
  float      m_convFactor1;  // conversion from binary ADC output to volts for pin1
  float      m_convFactor2;  // conversion from binary ADC output to volts for pin2
  uint8_t    m_channel;      // 0 or 1
  enStatus   m_status;
  uint32_t   m_period;
  uint32_t   m_endTime;

  uint32_t   m_val;
  float      m_factor;

  void StartConversion( uint8_t mux );
  uint32_t ReadResult();

  inline bool IsReady();
};
