/* 
  TinyGpsWrapper for Teensy 3.x
  -------------------------------------------------------------
  
  Copyright (C) 2015 Bernd Wokoeck
  
  Version history:
  1.00   02/21/2016  created

  Dependencies:
  - TinyGPS
  
**************************************************************/

#include "GpsSensor.h"

void GpsSensor::Init( enGpsSerial comPort )
{
  switch( comPort )
  {
  default:
  case SERIAL3: m_pSerial = &Serial3; break;
  case SERIAL2: m_pSerial = &Serial2; break;
  case SERIAL1: m_pSerial = &Serial1; break;
  }

  m_pSerial->begin(9600);
  delay(1000);
}

void GpsSensor::DoGpsSensor()
{
  if( m_pSerial )
  {
    while( m_pSerial->available() )
    {
      char c = m_pSerial->read();
      // Serial.print( c ); 
      if( m_gps.encode( c ) )
      {
        m_gps.f_get_position( &m_flat, &m_flon, &m_age );
        m_speed = (long)m_gps.f_speed_kmph();  // km/h 
        m_alt   = m_gps.altitude() / 100;      // m
        m_hdop  = m_gps.hdop();
        break;
      }
    }
  }
}


long GpsSensor::GetFenceDist( float Flat, float Flon )
{
  // point A = first point od fence line
  // point B = second point od fence line
  // point F = object
  // point S = intersection point of orthogonal line on AB through F
  
  float dy_AB  = 111.3 * (m_Alat - m_Blat) * 1000;
  float dx_AB  =  71.5 * (m_Alon - m_Blon) * 1000;
  float distAB = sqrt( dy_AB*dy_AB + dx_AB*dx_AB ); // seite a

  float dy_AF  = 111.3 * (m_Alat - Flat) * 1000;
  float dx_AF  =  71.5 * (m_Alon - Flon) * 1000;
  float distAF = sqrt( dy_AF*dy_AF + dx_AF*dx_AF ); // seite b

  float dy_BF  = 111.3 * (m_Blat - Flat) * 1000;
  float dx_BF  =  71.5 * (m_Blon - Flon) * 1000;
  float distBF = sqrt( dy_BF*dy_BF + dx_BF*dx_BF ); // seite c

  float s = 0.5f * (distAB + distAF + distBF); // s = 1/2(a+b+c)

  // ha=(2/a)sqrt[s(s-a)(s-b)(s-c)]
  float ha = (2/distAB) * sqrt( s*(s-distAB) * (s-distAF) * (s-distBF)); // absolute distance to fence line 

  // Sign
  float distBS   = sqrt( distBF*distBF - ha*ha );
  float divideAB = distBS / distAB;

  // float Slat = m_Blat+(m_Alat-m_Blat)*divideAB;
  float Slon = m_Blon+(m_Alon-m_Blon)*divideAB;
  float dx_FS = Flon - Slon;

  if( dx_FS > 0 )
    return ha;
  else
    return -ha;
}
