/* 
  TinyGpsWrapper for Teensy 3.x
  -------------------------------------------------------------
  
  Copyright (C) 2015 Bernd Wokoeck
  
  Version history:
  1.00   02/21/2016  created

  Dependencies:
  - TinyGPS
  
**************************************************************/
#if ARDUINO >= 100
 #include "Arduino.h"
#else
 #include "WProgram.h"
#endif

#include <TinyGPS.h>

class GpsSensor
{
public:
  GpsSensor() : m_pSerial( 0 ), m_flat( 0.0f), m_flon( 0.0f ), m_alt( 0 ), m_speed( 0 ), m_hdop( 0 ), m_age( 0 ) {}
  ~GpsSensor() {}

  enum enGpsSerial
  {
    SERIAL1 = 0x01,
    SERIAL2 = 0x02,
    SERIAL3 = 0x03,
  };

  void Init( enGpsSerial comPort );
  void DoGpsSensor();

  float GetLat()   { return m_flat; }
  float GetLon()   { return m_flon; }
  long  GetAlt()   { return m_alt; }         // in m
  long  GetSpeed() { return m_speed; }       // in km/h
  long  GetHDOP()  { return (long)m_hdop; }  // in 100th

  // fence function
  void SetFencePoints( float Alat, float Alon, float Blat, float Blon ) { m_Alat = Alat; m_Alon = Alon, m_Blat = Blat; m_Blon = Blon; }
  long GetFenceDist( float Flat, float Flon ); // shortest distance to fence line in m

protected:

  TinyGPS          m_gps;
  HardwareSerial * m_pSerial;

  float    m_flat;
  float    m_flon;
  long     m_alt;
  long     m_speed;
  uint32_t m_hdop;
  uint32_t m_age;

  // fence data
  float m_Alat;
  float m_Alon;
  float m_Blat;
  float m_Blon;
};
